@@ -6,6 +6,7 @@ use crate::ws::Msg::Close;
 use byteorder::{ByteOrder, BigEndian, NetworkEndian};
 use crate::deflate::Inflator;
 use async_channel::{Sender, Receiver, RecvError};
+use simdutf8::basic::from_utf8;
 
 #[derive(Debug)]
 // Message for communication with upstream part of the library.
@@ -47,7 +48,9 @@ impl Msg {
 pub struct Reader<R> {
     stream_rx: R,
     tx: Sender<Msg>,
+    sender: Sender<Msg>,
     deflate_supported: bool,
+    is_utf8_validate: bool,
     inflator: Inflator,
     payload: [u8; 1024],
     max_payload_len: usize,
@@ -60,13 +63,15 @@ impl<R> Reader<R>
     where
         R: AsyncReadExt + std::marker::Unpin + std::marker::Send + 'static,
 {
-    pub fn spawn(stream_rx: R, deflate_supported: bool) -> Receiver<Msg>
+    pub fn spawn(stream_rx: R, sender: Sender<Msg>, deflate_supported: bool, is_utf8_validate: bool) -> Receiver<Msg>
     {
         let (tx, rx): (Sender<Msg>, Receiver<Msg>) = async_channel::bounded::<Msg>(25);
         let mut reader = Reader {
             stream_rx,
             tx, // output of the messages to the application
+            sender,
             deflate_supported,
+            is_utf8_validate,
             inflator: Inflator::new(),
             payload: [0u8; 1024],
             max_payload_len: 1024,
@@ -197,8 +202,10 @@ impl<R> Reader<R>
                 }
             }
 
-            if !self.frame.validate_payload() {
-                break;
+            if self.is_utf8_validate {
+                if !self.frame.validate_payload() {
+                    break;
+                }
             }
 
             // unsafe {
@@ -216,6 +223,7 @@ impl<R> Reader<R>
             } else if self.frame.is_binary() {
                 self.tx.send(Msg::Binary(self.frame.payload.clone())).await;
             } else if self.frame.is_ping() {
+                self.sender.send(Msg::Pong(self.frame.payload.clone())).await;
                 self.tx.send(Msg::Ping(self.frame.payload.clone())).await;
             } else if self.frame.is_close() {
                 let status = self.frame.status();
@@ -358,11 +366,11 @@ impl Frame {
         self.rsv == 0
     }
 
-    fn validate_payload(&mut self) -> bool {
+    fn validate_payload(&self) -> bool {
         if !self.is_text() {
             return true;
         }
-        let res = String::from_utf8(self.payload.to_vec());
+        let res = from_utf8(self.payload.as_slice());
         if res.is_err() {
             error!("from_utf8 {}", res.unwrap_err());
             return false;
